use ::libc;
extern "C" {
    #[no_mangle]
    fn malloc(_: libc::c_ulong) -> *mut libc::c_void;
    #[no_mangle]
    fn free(__ptr: *mut libc::c_void);
    #[no_mangle]
    fn pthread_mutex_init(__mutex: *mut pthread_mutex_t,
                          __mutexattr: *const pthread_mutexattr_t)
     -> libc::c_int;
    #[no_mangle]
    fn pthread_mutex_destroy(__mutex: *mut pthread_mutex_t) -> libc::c_int;
    #[no_mangle]
    fn pthread_mutex_lock(__mutex: *mut pthread_mutex_t) -> libc::c_int;
    #[no_mangle]
    fn pthread_mutex_unlock(__mutex: *mut pthread_mutex_t) -> libc::c_int;
    #[no_mangle]
    fn pthread_mutexattr_init(__attr: *mut pthread_mutexattr_t)
     -> libc::c_int;
    #[no_mangle]
    fn pthread_mutexattr_destroy(__attr: *mut pthread_mutexattr_t)
     -> libc::c_int;
    #[no_mangle]
    fn pthread_mutexattr_setprotocol(__attr: *mut pthread_mutexattr_t,
                                     __protocol: libc::c_int) -> libc::c_int;
}
pub type size_t = libc::c_ulong;
pub type __uint8_t = libc::c_uchar;
pub type u_int8_t = __uint8_t;
#[derive(Copy, Clone)]
#[repr(C)]
pub struct __pthread_internal_list {
    pub __prev: *mut __pthread_internal_list,
    pub __next: *mut __pthread_internal_list,
}
pub type __pthread_list_t = __pthread_internal_list;
#[derive(Copy, Clone)]
#[repr(C)]
pub struct __pthread_mutex_s {
    pub __lock: libc::c_int,
    pub __count: libc::c_uint,
    pub __owner: libc::c_int,
    pub __nusers: libc::c_uint,
    pub __kind: libc::c_int,
    pub __spins: libc::c_short,
    pub __elision: libc::c_short,
    pub __list: __pthread_list_t,
}
#[derive(Copy, Clone)]
#[repr(C)]
pub union pthread_mutexattr_t {
    pub __size: [libc::c_char; 4],
    pub __align: libc::c_int,
}
#[derive(Copy, Clone)]
#[repr(C)]
pub union pthread_mutex_t {
    pub __data: __pthread_mutex_s,
    pub __size: [libc::c_char; 40],
    pub __align: libc::c_long,
}
pub type C2RustUnnamed = libc::c_uint;
pub const PTHREAD_PRIO_PROTECT: C2RustUnnamed = 2;
pub const PTHREAD_PRIO_INHERIT: C2RustUnnamed = 1;
pub const PTHREAD_PRIO_NONE: C2RustUnnamed = 0;
pub type u8_t = u_int8_t;
#[derive(Copy, Clone)]
#[repr(C)]
pub struct buffer {
    pub buf: *mut u8_t,
    pub readp: *mut u8_t,
    pub writep: *mut u8_t,
    pub wrap: *mut u8_t,
    pub size: size_t,
    pub base_size: size_t,
    pub mutex: pthread_mutex_t,
}
/* 
 *  Squeezelite - lightweight headless squeezebox emulator
 *
 *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
 *      Ralph Irving 2015-2017, ralph_irving@hotmail.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
// fifo bufffers 
// _* called with muxtex locked
#[inline]
unsafe extern "C" fn _buf_used(mut buf: *mut buffer) -> libc::c_uint {
    return if (*buf).writep >= (*buf).readp {
               (*buf).writep.wrapping_offset_from((*buf).readp) as
                   libc::c_long as libc::c_ulong
           } else {
               (*buf).size.wrapping_sub((*buf).readp.wrapping_offset_from((*buf).writep)
                                            as libc::c_long as libc::c_ulong)
           } as libc::c_uint;
}
#[no_mangle]
pub unsafe extern "C" fn _buf_space(mut buf: *mut buffer) -> libc::c_uint {
    return (*buf).size.wrapping_sub(_buf_used(buf) as
                                        libc::c_ulong).wrapping_sub(1 as
                                                                        libc::c_int
                                                                        as
                                                                        libc::c_ulong)
               as libc::c_uint;
    // reduce by one as full same as empty otherwise
}
#[no_mangle]
pub unsafe extern "C" fn _buf_cont_read(mut buf: *mut buffer)
 -> libc::c_uint {
    return if (*buf).writep >= (*buf).readp {
               (*buf).writep.wrapping_offset_from((*buf).readp) as
                   libc::c_long
           } else {
               (*buf).wrap.wrapping_offset_from((*buf).readp) as libc::c_long
           } as libc::c_uint;
}
#[no_mangle]
pub unsafe extern "C" fn _buf_cont_write(mut buf: *mut buffer)
 -> libc::c_uint {
    return if (*buf).writep >= (*buf).readp {
               (*buf).wrap.wrapping_offset_from((*buf).writep) as libc::c_long
           } else {
               (*buf).readp.wrapping_offset_from((*buf).writep) as
                   libc::c_long
           } as libc::c_uint;
}
#[no_mangle]
pub unsafe extern "C" fn _buf_inc_readp(mut buf: *mut buffer,
                                        mut by: libc::c_uint) {
    (*buf).readp = (*buf).readp.offset(by as isize);
    if (*buf).readp >= (*buf).wrap {
        (*buf).readp = (*buf).readp.offset(-((*buf).size as isize))
    };
}
#[no_mangle]
pub unsafe extern "C" fn _buf_inc_writep(mut buf: *mut buffer,
                                         mut by: libc::c_uint) {
    (*buf).writep = (*buf).writep.offset(by as isize);
    if (*buf).writep >= (*buf).wrap {
        (*buf).writep = (*buf).writep.offset(-((*buf).size as isize))
    };
}
#[no_mangle]
pub unsafe extern "C" fn buf_flush(mut buf: *mut buffer) {
    pthread_mutex_lock(&mut (*buf).mutex);
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    pthread_mutex_unlock(&mut (*buf).mutex);
}
// adjust buffer to multiple of mod bytes so reading in multiple always wraps on frame boundary
#[no_mangle]
pub unsafe extern "C" fn buf_adjust(mut buf: *mut buffer, mut mod_0: size_t) {
    let mut size: size_t = 0;
    pthread_mutex_lock(&mut (*buf).mutex);
    size =
        ((*buf).base_size.wrapping_div(mod_0) as libc::c_uint as
             libc::c_ulong).wrapping_mul(mod_0);
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    (*buf).wrap = (*buf).buf.offset(size as isize);
    (*buf).size = size;
    pthread_mutex_unlock(&mut (*buf).mutex);
}
// called with mutex locked to resize, does not retain contents, reverts to original size if fails
#[no_mangle]
pub unsafe extern "C" fn _buf_resize(mut buf: *mut buffer, mut size: size_t) {
    free((*buf).buf as *mut libc::c_void);
    (*buf).buf = malloc(size) as *mut u8_t;
    if (*buf).buf.is_null() {
        size = (*buf).size;
        (*buf).buf = malloc(size) as *mut u8_t;
        if (*buf).buf.is_null() { size = 0 as libc::c_int as size_t }
    }
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    (*buf).wrap = (*buf).buf.offset(size as isize);
    (*buf).size = size;
    (*buf).base_size = size;
}
#[no_mangle]
pub unsafe extern "C" fn buf_init(mut buf: *mut buffer, mut size: size_t) {
    (*buf).buf = malloc(size) as *mut u8_t;
    (*buf).readp = (*buf).buf;
    (*buf).writep = (*buf).buf;
    (*buf).wrap = (*buf).buf.offset(size as isize);
    (*buf).size = size;
    (*buf).base_size = size;
    let mut attr: pthread_mutexattr_t = pthread_mutexattr_t{__size: [0; 4],};
    pthread_mutexattr_init(&mut attr);
    pthread_mutexattr_setprotocol(&mut attr,
                                  PTHREAD_PRIO_INHERIT as libc::c_int);
    pthread_mutex_init(&mut (*buf).mutex, &mut attr);
    pthread_mutexattr_destroy(&mut attr);
}
/* 
 *  Squeezelite - lightweight headless squeezebox emulator
 *
 *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
 *      Ralph Irving 2015-2017, ralph_irving@hotmail.com
 *  
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additions (c) Paul Hermann, 2015-2017 under the same license terms
 *   -Control of Raspberry pi GPIO for amplifier power
 *   -Launch script on power status change from LMS
 */
// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, RESAMPLE_MP, VISEXPORT, GPIO, IR, DSD, LINKALL to influence build
// build detection
// dynamically loaded libraries at run time
// !LINKALL
// config options
// do not reduce as icy-meta max is 4080
/* SUN */
/* SUN */
// printf/scanf formats for u64_t
// logging
// utils.c (non logging)
// buffer.c
// _* called with mutex locked
#[no_mangle]
pub unsafe extern "C" fn buf_destroy(mut buf: *mut buffer) {
    if !(*buf).buf.is_null() {
        free((*buf).buf as *mut libc::c_void);
        (*buf).buf = 0 as *mut u8_t;
        (*buf).size = 0 as libc::c_int as size_t;
        (*buf).base_size = 0 as libc::c_int as size_t;
        pthread_mutex_destroy(&mut (*buf).mutex);
    };
}
